@@ -37,7 +37,7 @@
 extern "C" {
 #endif
 
-#define LIBGNUTLS_VERSION "0.9.94"
+#define LIBGNUTLS_VERSION "0.9.95"
 
 /* For backwards compatibility */
 #include <gnutls/compat4.h>
@@ -106,11 +106,23 @@ typedef enum gnutls_handshake_description { GNUTLS_HANDSHAKE_HELLO_REQUEST,
 	GNUTLS_HANDSHAKE_FINISHED=20 
 } gnutls_handshake_description;
 
+/* Note that the status bits have different meanings
+ * in openpgp keys and x.509 certificate verification.
+ */
+#define GNUTLS_CERT_NOT_TRUSTED GNUTLS_CERT_INVALID
 typedef enum gnutls_certificate_status { 
-	GNUTLS_CERT_NOT_TRUSTED=2, 
-	GNUTLS_CERT_INVALID=4, 
-	GNUTLS_CERT_CORRUPTED=16,
-	GNUTLS_CERT_REVOKED=32
+	GNUTLS_CERT_INVALID=2, /* will be set if the certificate
+				* was not verified.
+				*/
+	GNUTLS_CERT_REVOKED=32, /* in X.509 this will be set only if CRLs are checked
+				 */
+
+	/* Those are extra information about the verification
+	 * process. Will be set only if the certificate was 
+	 * not verified.
+	 */
+	GNUTLS_CERT_SIGNER_NOT_FOUND=64,
+	GNUTLS_CERT_SIGNER_NOT_CA=128
 } gnutls_certificate_status;
 
 typedef enum gnutls_certificate_request { GNUTLS_CERT_IGNORE, GNUTLS_CERT_REQUEST=1, GNUTLS_CERT_REQUIRE } gnutls_certificate_request;
@@ -152,6 +164,11 @@ typedef struct {
 	unsigned int size;
 } gnutls_datum;
 
+typedef struct {
+	const unsigned char * data;
+	unsigned int size;
+} gnutls_const_datum;
+
 /* internal functions */
 
 int gnutls_init(gnutls_session * session, gnutls_connection_end con_end);
@@ -640,6 +657,9 @@ int gnutls_pem_base64_decode_alloc( const char* header, const gnutls_datum *b64_
 
 #define GNUTLS_E_BASE64_ENCODING_ERROR -201
 #define GNUTLS_E_INCOMPATIBLE_GCRYPT_LIBRARY -202
+#define GNUTLS_E_INCOMPATIBLE_LIBTASN1_LIBRARY -203
+
+#define GNUTLS_E_OPENPGP_KEYRING_ERROR -204
 
 #define GNUTLS_E_UNIMPLEMENTED_FEATURE -1250
 
