@@ -31,7 +31,7 @@
 #include <gnutls_errors.h>
 #include <common.h>
 #include <pkcs12.h>
-
+#include <privkey.h>
 
 /**
   * gnutls_pkcs12_bag_init - This function initializes a gnutls_pkcs12_bag structure
@@ -118,12 +118,12 @@ int gnutls_pkcs12_bag_get_count(gnutls_pkcs12_bag bag)
   * gnutls_pkcs12_bag_get_data - This function returns the bag's data
   * @bag: The bag
   * @indx: The element of the bag to get the data from
-  * @data: where the data will be copied to. Should be treated as constant.
+  * @data: where the bag's data will be. Should be treated as constant.
   *
   * This function will return the bag's data. 
   *
   **/
-int gnutls_pkcs12_bag_get_data(gnutls_pkcs12_bag bag, int indx, gnutls_datum* data)
+int gnutls_pkcs12_bag_get_data(gnutls_pkcs12_bag bag, int indx, gnutls_const_datum * data)
 {
 	if (indx >= bag->bag_elements) 
 		return GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
@@ -315,6 +315,66 @@ int ret;
 	return bag->bag_elements-1;
 }
 
+/**
+  * gnutls_pkcs12_bag_set_crt - This function inserts a certificate into the bag
+  * @bag: The bag
+  * @crt: the certificate to be copied.
+  *
+  * This function will insert the given certificate into the
+  * bag. This is just a wrapper over gnutls_pkcs12_bag_set_data().
+  *
+  * On success returns the index of the added bag, or a negative
+  * value on error.
+  *
+  **/
+int gnutls_pkcs12_bag_set_crt(gnutls_pkcs12_bag bag, gnutls_x509_crt crt)
+{
+int ret;
+gnutls_datum data;
+
+	ret = _gnutls_x509_der_encode( crt->cert, "", &data, 0);
+	if (ret < 0) {
+		gnutls_assert();
+		return ret;
+	}
+	
+	ret = gnutls_pkcs12_bag_set_data( bag, GNUTLS_BAG_CERTIFICATE, &data);
+
+	_gnutls_free_datum( &data);
+	
+	return ret;
+}
+
+/**
+  * gnutls_pkcs12_bag_set_crl - This function inserts the CRL into the bag
+  * @bag: The bag
+  * @crl: the CRL to be copied.
+  *
+  * This function will insert the given CRL into the
+  * bag. This is just a wrapper over gnutls_pkcs12_bag_set_data().
+  *
+  * On success returns the index of the added bag, or a negative
+  * value on error.
+  *
+  **/
+int gnutls_pkcs12_bag_set_crl(gnutls_pkcs12_bag bag, gnutls_x509_crl crl)
+{
+int ret;
+gnutls_datum data;
+
+	ret = _gnutls_x509_der_encode( crl->crl, "", &data, 0);
+	if (ret < 0) {
+		gnutls_assert();
+		return ret;
+	}
+	
+	ret = gnutls_pkcs12_bag_set_data( bag, GNUTLS_BAG_CRL, &data);
+
+	_gnutls_free_datum( &data);
+	
+	return ret;
+}
+
 /**
   * gnutls_pkcs12_bag_set_key_id - This function sets a key ID into the bag element
   * @bag: The bag
@@ -482,7 +542,7 @@ gnutls_datum dec;
   * gnutls_pkcs12_bag_encrypt - This function will encrypt a bag
   * @bag: The bag
   * @pass: The password used for encryption
-  * @flags: should be zero for now
+  * @flags: should be one of gnutls_pkcs_encrypt_flags elements bitwise or'd
   *
   * This function will encrypt the given bag and return 0 on success.
   *
@@ -493,6 +553,7 @@ int ret;
 ASN1_TYPE safe_cont = ASN1_TYPE_EMPTY;
 gnutls_datum der = {NULL, 0};
 gnutls_datum enc = {NULL, 0};
+schema_id id;
 
 	if (bag->element[0].type == GNUTLS_BAG_ENCRYPTED) {
 		gnutls_assert();
@@ -519,9 +580,19 @@ gnutls_datum enc = {NULL, 0};
         	return ret;
         }
 
+	if (flags & GNUTLS_PKCS_PLAIN) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+	
+	if (flags & GNUTLS_PKCS_USE_PKCS12_ARCFOUR) id = PKCS12_ARCFOUR_SHA1;
+	else if (flags & GNUTLS_PKCS_USE_PKCS12_RC2_40) id = PKCS12_RC2_40_SHA1;
+	else if (flags & GNUTLS_PKCS_USE_PBES2_3DES) id = PBES2;
+	else id = PKCS12_3DES_SHA1;
+
 	/* Now encrypt them.
 	 */
-	ret = _gnutls_pkcs7_encrypt_data( PKCS12_3DES_SHA1, &der, pass, &enc);
+	ret = _gnutls_pkcs7_encrypt_data( id, &der, pass, &enc);
 
 	_gnutls_free_datum( &der);
 
