@@ -40,7 +40,7 @@
 
 static int _gnutls_verify_certificate2(gnutls_x509_crt cert,
 			       gnutls_x509_crt *trusted_cas, int tcas_size, 
-			       unsigned int flags);
+			       unsigned int flags, unsigned int* output);
 static
 int _gnutls_x509_verify_signature(const gnutls_datum* signed_data,
 	const gnutls_datum* signature, gnutls_x509_crt issuer);
@@ -48,7 +48,7 @@ static
 int is_crl_issuer(gnutls_x509_crl crl, gnutls_x509_crt issuer_cert);
 static int _gnutls_verify_crl2(gnutls_x509_crl crl,
 			       gnutls_x509_crt *trusted_cas, int tcas_size, 
-			       unsigned int flags);
+			       unsigned int flags, unsigned int *output);
 
 
 /* Checks if the issuer of a certificate is a
@@ -184,20 +184,26 @@ gnutls_x509_crt find_issuer(gnutls_x509_crt cert,
  * was successfuly verified.
  *
  * 'flags': an OR of the gnutls_certificate_verify_flags enumeration.
+ *
+ * Output will hold some extra information about the verification
+ * procedure.
  */
 static int _gnutls_verify_certificate2(gnutls_x509_crt cert,
 			       gnutls_x509_crt *trusted_cas, int tcas_size, 
-			       unsigned int flags)
+			       unsigned int flags, unsigned int *output)
 {
 gnutls_datum cert_signed_data = { NULL, 0 };
 gnutls_datum cert_signature = { NULL, 0 };
 gnutls_x509_crt issuer;
 int ret, issuer_version, result;
 
+	if (output) *output = 0;
+
 	if (tcas_size >= 1)
 		issuer = find_issuer(cert, trusted_cas, tcas_size);
 	else {
 		gnutls_assert();
+		if (output) *output |= GNUTLS_CERT_SIGNER_NOT_FOUND | GNUTLS_CERT_INVALID;
 		return 0;
 	}
 
@@ -205,6 +211,7 @@ int ret, issuer_version, result;
 	 * authorities.
 	 */
 	if (issuer == NULL) {
+		if (output) *output |= GNUTLS_CERT_SIGNER_NOT_FOUND | GNUTLS_CERT_INVALID;
 		gnutls_assert();
 		return 0;
 	}
@@ -219,6 +226,7 @@ int ret, issuer_version, result;
 		!((flags & GNUTLS_VERIFY_ALLOW_X509_V1_CA_CRT) && issuer_version == 1)) {
 		if (check_if_ca(cert, issuer)==0) {
 			gnutls_assert();
+			if (output) *output |= GNUTLS_CERT_SIGNER_NOT_CA | GNUTLS_CERT_INVALID;
 			return 0;
 		}
 	}
@@ -239,7 +247,10 @@ int ret, issuer_version, result;
 	ret = _gnutls_x509_verify_signature(&cert_signed_data, &cert_signature, issuer);
 	if (ret < 0) {
 		gnutls_assert();
+	} else if (ret == 0) {
+		gnutls_assert();
 		/* error. ignore it */
+		if (output) *output |= GNUTLS_CERT_INVALID;
 		ret = 0;
 	}
 
@@ -296,7 +307,7 @@ unsigned int _gnutls_x509_verify_certificate(gnutls_x509_crt * certificate_list,
 
 		if ((ret =
 		     _gnutls_verify_certificate2(certificate_list[i],
-						 &certificate_list[i + 1], 1, flags)) != 1) 
+						 &certificate_list[i + 1], 1, flags, NULL)) != 1) 
 		{
 			status |= GNUTLS_CERT_INVALID;
 		}
@@ -308,7 +319,7 @@ unsigned int _gnutls_x509_verify_certificate(gnutls_x509_crt * certificate_list,
 		   * and return immediately.
 		   */
 		gnutls_assert();
-		return (status | GNUTLS_CERT_NOT_TRUSTED);
+		return (0 | GNUTLS_CERT_INVALID);
 	}
 	
 	/* Now verify the last certificate in the certificate path
@@ -319,7 +330,7 @@ unsigned int _gnutls_x509_verify_certificate(gnutls_x509_crt * certificate_list,
 	 */
 	ret =
 	    _gnutls_verify_certificate2(certificate_list[i], trusted_cas,
-				       tcas_size, flags);
+				       tcas_size, flags, NULL);
 
 	if (ret == 0) {
 		/* if the last certificate in the certificate
@@ -327,7 +338,7 @@ unsigned int _gnutls_x509_verify_certificate(gnutls_x509_crt * certificate_list,
 		 * trusted.
 		 */
 		gnutls_assert();
-		status |= GNUTLS_CERT_NOT_TRUSTED;
+		status |= GNUTLS_CERT_INVALID;
 	}
 
 	return status;
@@ -522,29 +533,25 @@ int ret, issuer_params_size, i;
   * @flags: Flags that may be used to change the verification algorithm. Use OR of the gnutls_certificate_verify_flags enumerations.
   * @verify: will hold the certificate verification output.
   *
-  * This function will try to verify the given certificate list and return its status (TRUSTED, REVOKED etc.). 
-  * The return value (status) should be one or more of the gnutls_certificate_status 
-  * enumerated elements bitwise or'd. Note that expiration and activation dates are not checked 
+  * This function will try to verify the given certificate list and return its status.
+  * Note that expiration and activation dates are not checked
   * by this function, you should check them using the appropriate functions.
   *
   * If no flags are specified (0), this function will use the 
   * basicConstraints (2.5.29.19) PKIX extension. This means that only a certificate 
   * authority is allowed to sign a certificate.
   *
-  * However you must also check the peer's name in order to check if the verified 
+  * You must also check the peer's name in order to check if the verified 
   * certificate belongs to the actual peer. 
   *
-  *
   * The certificate verification output will be put in 'verify' and will be
   * one or more of the gnutls_certificate_status enumerated elements bitwise or'd.
+  * For a more detailed verification status use gnutls_x509_crt_verify() per list
+  * element.
   *
-  * GNUTLS_CERT_NOT_TRUSTED\: the peer's certificate is not trusted.
+  * GNUTLS_CERT_INVALID\: the certificate chain is not valid.
   *
-  * GNUTLS_CERT_INVALID\: the certificate chain is broken.
-  *
-  * GNUTLS_CERT_REVOKED\: the certificate has been revoked.
-  *
-  * GNUTLS_CERT_CORRUPTED\: the certificate is corrupted.
+  * GNUTLS_CERT_REVOKED\: a certificate in the chain has been revoked.
   *
   * Returns 0 on success and a negative value in case of an error.
   *
@@ -575,8 +582,7 @@ int gnutls_x509_crt_list_verify( gnutls_x509_crt* cert_list, int cert_list_lengt
   * @verify: will hold the certificate verification output.
   *
   * This function will try to verify the given certificate and return its status. 
-  * See gnutls_x509_crt_list_verify() for a detailed description of
-  * return values.
+  * The verification output in this functions cannot be GNUTLS_CERT_NOT_VALID.
   *
   * Returns 0 on success and a negative value in case of an error.
   *
@@ -585,10 +591,14 @@ int gnutls_x509_crt_verify( gnutls_x509_crt cert,
 	gnutls_x509_crt *CA_list, int CA_list_length,
 	unsigned int flags, unsigned int *verify)
 {
+int ret;
 	/* Verify certificate 
 	 */
-	*verify =
-	    _gnutls_verify_certificate2( cert, CA_list, CA_list_length, flags);
+	ret = _gnutls_verify_certificate2( cert, CA_list, CA_list_length, flags, verify);
+	if (ret < 0) {
+		gnutls_assert();
+		return ret;
+	}
 
 	return 0;
 }
@@ -651,10 +661,14 @@ int gnutls_x509_crl_verify( gnutls_x509_crl crl,
 	gnutls_x509_crt *CA_list, int CA_list_length,
 	unsigned int flags, unsigned int *verify)
 {
+int ret;
 	/* Verify crl 
 	 */
-	*verify =
-	    _gnutls_verify_crl2( crl, CA_list, CA_list_length, flags);
+	ret = _gnutls_verify_crl2( crl, CA_list, CA_list_length, flags, verify);
+	if (ret < 0) {
+		gnutls_assert();
+		return ret;
+	}
 
 	return 0;
 }
@@ -712,10 +726,13 @@ gnutls_x509_crt find_crl_issuer(gnutls_x509_crl crl,
  * was successfuly verified.
  *
  * 'flags': an OR of the gnutls_certificate_verify_flags enumeration.
+ *
+ * Output will hold information about the verification
+ * procedure. 
  */
 static int _gnutls_verify_crl2(gnutls_x509_crl crl,
 			       gnutls_x509_crt *trusted_cas, int tcas_size, 
-			       unsigned int flags)
+			       unsigned int flags, unsigned int* output)
 {
 /* CRL is ignored for now */
 gnutls_datum crl_signed_data = { NULL, 0 };
@@ -723,10 +740,13 @@ gnutls_datum crl_signature = { NULL, 0 };
 gnutls_x509_crt issuer;
 int ret, result;
 
+	if (output) *output = 0;
+
 	if (tcas_size >= 1)
 		issuer = find_crl_issuer(crl, trusted_cas, tcas_size);
 	else {
 		gnutls_assert();
+		if (output) *output |= GNUTLS_CERT_SIGNER_NOT_FOUND | GNUTLS_CERT_INVALID;
 		return 0;
 	}
 
@@ -735,6 +755,7 @@ int ret, result;
 	 */
 	if (issuer == NULL) {
 		gnutls_assert();
+		if (output) *output |= GNUTLS_CERT_SIGNER_NOT_FOUND | GNUTLS_CERT_INVALID;
 		return 0;
 	}
 
@@ -742,6 +763,7 @@ int ret, result;
 		if (gnutls_x509_crt_get_ca_status(issuer, NULL) != 1) 
 		{
 			gnutls_assert();
+			if (output) *output |= GNUTLS_CERT_SIGNER_NOT_CA | GNUTLS_CERT_INVALID;
 			return 0;
 		}
 	}
@@ -758,11 +780,13 @@ int ret, result;
 		goto cleanup;
 	}
 
-
 	ret = _gnutls_x509_verify_signature(&crl_signed_data, &crl_signature, issuer);
 	if (ret < 0) {
 		gnutls_assert();
+	} else if (ret == 0) {
+		gnutls_assert();
 		/* error. ignore it */
+		if (output) *output |= GNUTLS_CERT_INVALID;
 		ret = 0;
 	}
 
