@@ -162,8 +162,16 @@ int _gnutls_base64_encode(const uint8 * data, size_t data_size, uint8 ** result)
 	return ret;
 }
 
+#define INCR(what, size) \
+	what+=size; \
+	if (what > ret) { \
+		gnutls_assert(); \
+		gnutls_free( (*result)); *result = NULL; \
+		return GNUTLS_E_INTERNAL_ERROR; \
+	}
+
 /* encodes data and puts the result into result (locally allocated)
- * The result_size is the return value
+ * The result_size (including the null terminator) is the return value.
  */
 int _gnutls_fbase64_encode(const char *msg, const uint8 * data, int data_size,
 			   uint8 ** result)
@@ -173,11 +181,13 @@ int _gnutls_fbase64_encode(const char *msg, const uint8 * data, int data_size,
 	uint8 *ptr;
 	uint8 top[80];
 	uint8 bottom[80];
-	int pos;
+	int pos, bytes, top_len, bottom_len;
 	size_t msglen = strlen(msg);
 
-	if (msglen > 50)
+	if (msglen > 50) {
+		gnutls_assert();
 		return GNUTLS_E_BASE64_ENCODING_ERROR;
+	}
 
 	memset(bottom, 0, sizeof(bottom));
 	memset(top, 0, sizeof(top));
@@ -190,50 +200,70 @@ int _gnutls_fbase64_encode(const char *msg, const uint8 * data, int data_size,
 	strcat(bottom, msg); /* Flawfinder: ignore */
 	strcat(bottom, "-----\n"); /* Flawfinder: ignore */
 
+	top_len = strlen(top);
+	bottom_len = strlen(bottom);
+
 	ret = B64FSIZE( msglen, data_size);
 
 	(*result) = gnutls_calloc(1, ret + 1);
-	if ((*result) == NULL)
+	if ((*result) == NULL) {
+		gnutls_assert();
 		return GNUTLS_E_MEMORY_ERROR;
+	}
+
+	bytes = pos = 0;
+	INCR( bytes, top_len);
+	pos = top_len;
 
 	strcpy(*result, top); /* Flawfinder: ignore */
-	pos = strlen(top);
 
 	for (i = j = 0; i < data_size; i += 3, j += 4) {
+
 		tmp = encode(tmpres, &data[i], data_size - i);
 		if (tmp == -1) {
+			gnutls_assert();
 			gnutls_free( (*result)); *result = NULL;
 			return GNUTLS_E_BASE64_ENCODING_ERROR;
 		}
+
+		INCR(bytes, 4);
 		ptr = &(*result)[j + pos];
 
 		if ((j) % 64 == 0) {
+			INCR(bytes, 1);
 			pos++;
 			*ptr++ = '\n';
 		}
 		*ptr++ = tmpres[0];
 
 		if ((j + 1) % 64 == 0) {
-			*ptr++ = '\n';
+			INCR(bytes, 1);
 			pos++;
+			*ptr++ = '\n';
 		}
 		*ptr++ = tmpres[1];
 
 		if ((j + 2) % 64 == 0) {
+			INCR(bytes, 1);
 			pos++;
 			*ptr++ = '\n';
 		}
 		*ptr++ = tmpres[2];
 
 		if ((j + 3) % 64 == 0) {
-			*ptr++ = '\n';
+			INCR(bytes, 1);
 			pos++;
+			*ptr++ = '\n';
 		}
 		*ptr++ = tmpres[3];
 	}
 
-	strcat(*result, bottom); /* Flawfinder: ignore */
-	return ret;
+	INCR( bytes, bottom_len);
+
+	memcpy( &(*result)[ bytes-bottom_len], bottom, bottom_len);
+	(*result)[ bytes] = 0;
+
+	return ret + 1;
 }
 
 /**
@@ -246,6 +276,9 @@ int _gnutls_fbase64_encode(const char *msg, const uint8 * data, int data_size,
   * This function will convert the given data to printable data, using the base64 
   * encoding. This is the encoding used in PEM messages. If the provided
   * buffer is not long enough GNUTLS_E_SHORT_MEMORY_BUFFER is returned.
+  *
+  * The output string will be null terminated, although the size will not include
+  * the terminating null.
   * 
   **/
 int gnutls_pem_base64_encode( const char* msg, const gnutls_datum *data, char* result, int* result_size) {
@@ -263,7 +296,7 @@ int size;
 	} else {
 		memcpy( result, ret, size);
 		gnutls_free(ret);
-		*result_size = size;
+		*result_size = size - 1;
 	}
 
 	return 0;
@@ -296,7 +329,7 @@ int size;
 		return size;
 
 	result->data = ret;
-	result->size = size;
+	result->size = size - 1;
 	return 0;
 }
 
