@@ -1,13 +1,38 @@
+/*
+ * Copyright (C) 2003 Nikos Mavroyanopoulos
+ *
+ * This file is part of GNUTLS.
+ *
+ * GNUTLS is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * GNUTLS is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ */
+
+#include <config.h>
+#include <gnutls/gnutls.h>
+
+#ifdef ENABLE_PKI
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
 #include <time.h>
 #include "certtool-gaa.h"
 #include <gnutls/pkcs12.h>
 #include <unistd.h>
 
+int generate_prime(int bits);
 void pkcs12_info( void);
 void generate_pkcs12( void);
 void verify_chain(void);
@@ -28,8 +53,8 @@ FILE* infile;
 int in_cert_format;
 int out_cert_format;
 
-static unsigned char buffer[50*1024];
-static const int buffer_size = sizeof(buffer);
+unsigned char buffer[50*1024];
+const int buffer_size = sizeof(buffer);
 
 static void tls_log_func( int level, const char* str)
 {
@@ -179,8 +204,14 @@ int size, ret;
 			exit(1);
 		}
 	} else {
+		unsigned int flags;
+		
+		if (info.export) flags = GNUTLS_PKCS_USE_PKCS12_RC2_40;
+		else flags = GNUTLS_PKCS_USE_PKCS12_3DES;
+		if (info.pass == NULL) flags = GNUTLS_PKCS_PLAIN;
+
 		size = sizeof(buffer);
-		ret = gnutls_x509_privkey_export_pkcs8( key, out_cert_format, NULL, GNUTLS_PKCS8_PLAIN, buffer, &size);
+		ret = gnutls_x509_privkey_export_pkcs8( key, out_cert_format, info.pass, flags, buffer, &size);
 		if (ret < 0) {
 			fprintf(stderr, "privkey_export_pkcs8: %s\n", gnutls_strerror(ret));
 			exit(1);
@@ -463,6 +494,8 @@ void update_signed_certificate( void)
 
 void gaa_parser(int argc, char **argv)
 {
+int ret;
+
 	if (gaa(argc, argv, &info) != -1) {
 		fprintf(stderr,
 			"Error in the arguments. Use the --help or -h parameters to get more information.\n");
@@ -491,7 +524,11 @@ void gaa_parser(int argc, char **argv)
 	if (info.outcert_format) out_cert_format = GNUTLS_X509_FMT_DER;
 	else out_cert_format = GNUTLS_X509_FMT_PEM;
 
-	gnutls_global_init();
+	if ((ret=gnutls_global_init()) < 0) {
+		fprintf(stderr, "global_init: %s\n", gnutls_strerror(ret));
+		exit(1);
+	}
+
 	gnutls_global_set_log_function( tls_log_func);
 	gnutls_global_set_log_level(info.debug);
 
@@ -526,16 +563,17 @@ void gaa_parser(int argc, char **argv)
 		case 9:
 			pkcs12_info();
 			break;
+		case 10:
+			generate_prime( info.bits);
+			break;
+		default:
+			fprintf(stderr, "GnuTLS' certtool utility.\n");
+			fprintf(stderr, "Please use the --help to get help on this program.\n");
+			exit(0);
 	}
 	fclose(outfile);
 }
 
-void certtool_version(void)
-{
-	fprintf(stderr, "certtool, ");
-	fprintf(stderr, "version %s. Libgnutls %s.\n", LIBGNUTLS_VERSION,
-		gnutls_check_version(NULL));
-}
 
 const char* get_algorithm( int a) 
 {
@@ -747,7 +785,7 @@ void privkey_info( void)
 	if (!info.pkcs8) {
 		ret = gnutls_x509_privkey_import(key, &pem, in_cert_format);
 	} else {
-		ret = gnutls_x509_privkey_import_pkcs8(key, &pem, in_cert_format, NULL, GNUTLS_PKCS8_PLAIN);
+		ret = gnutls_x509_privkey_import_pkcs8(key, &pem, in_cert_format, info.pass, 0);
 	}
 
 	if (ret < 0) {
@@ -818,7 +856,7 @@ size_t size;
 		ret = gnutls_x509_privkey_import( key, &dat, in_cert_format);
 	else
 		ret = gnutls_x509_privkey_import_pkcs8( key, &dat, in_cert_format,
-			NULL, 0);
+			info.pass, 0);
 	
 	if (ret < 0) {
 		fprintf(stderr, "privkey_import: %s\n", gnutls_strerror(ret));
@@ -907,7 +945,7 @@ size_t size;
 		ret = gnutls_x509_privkey_import( key, &dat, in_cert_format);
 	else
 		ret = gnutls_x509_privkey_import_pkcs8( key, &dat, in_cert_format,
-			NULL, 0);
+			info.pass, 0);
 	
 	if (ret < 0) {
 		fprintf(stderr, "privkey_import: %s\n", gnutls_strerror(ret));
@@ -1081,22 +1119,26 @@ void generate_request(void)
 
 }
 
-
+static void print_verification_res( gnutls_x509_crt crt, gnutls_x509_crt issuer,
+	gnutls_x509_crl *crl_list, int crl_list_size);
 
 #define CERT_SEP "-----BEGIN CERT"
 #define CRL_SEP "-----BEGIN X509 CRL"
-
 int _verify_x509_mem( const char* cert, int cert_size)
 {
 	int siz, i;
 	const char *ptr;
 	int ret;
 	unsigned int output;
+	char name[256];
+	char issuer_name[256];
+	size_t name_size;
+	size_t issuer_name_size;
 	gnutls_datum tmp;
 	gnutls_x509_crt *x509_cert_list = NULL;
 	gnutls_x509_crl *x509_crl_list = NULL;
 	int x509_ncerts, x509_ncrls;
-	time_t now = time(0);
+	
 
 	/* Decode the CA certificate
 	 */
@@ -1177,14 +1219,59 @@ int _verify_x509_mem( const char* cert, int cert_size)
 			exit(1);
 		}
 		
-		/* Check expiration dates.
-		 */
-		if (gnutls_x509_crt_get_activation_time(x509_cert_list[i-1]) > now)
-			fprintf(stderr, "Warning: certificate %d has not been activated yet.\n", i);
-		if (gnutls_x509_crt_get_expiration_time(x509_cert_list[i-1]) < now)
-			fprintf(stderr, "Warning: certificate %d has been expired.\n", i);
 
-		/* now we move ptr after the pem header */
+		if (i-1 != 0) {
+			/* verify the previous certificate using this one 
+			 * as CA.
+			 */
+
+			name_size = sizeof(name);
+			ret = gnutls_x509_crt_get_dn( x509_cert_list[i-2], name, &name_size);
+			if (ret < 0) {
+				fprintf(stderr, "Error in get_dn: %s\n", gnutls_strerror(ret));
+				exit(1);
+			}
+
+			fprintf( outfile, "Certificate[%d]: %s\n", i-2, name);
+
+			/* print issuer 
+			 */
+			issuer_name_size = sizeof(issuer_name);
+			ret = gnutls_x509_crt_get_issuer_dn( x509_cert_list[i-2], issuer_name, &issuer_name_size);
+			if (ret < 0) {
+				fprintf(stderr, "Error in get_dn: %s\n", gnutls_strerror(ret));
+				exit(1);
+			}
+
+			fprintf( outfile, "\tIssued by: %s\n", name);
+			
+			/* Get the Issuer's name
+			 */
+			name_size = sizeof(name);
+			ret = gnutls_x509_crt_get_dn( x509_cert_list[i-1], name, &name_size);
+			if (ret < 0) {
+				fprintf(stderr, "Error in get_dn: %s\n", gnutls_strerror(ret));
+				exit(1);
+			}
+
+			fprintf( outfile, "\tVerifying against certificate[%d].\n", i-1);
+
+			if (strcmp( issuer_name, name) != 0) {
+				fprintf(stderr, "Error: Issuer's name: %s\n", name);
+				fprintf(stderr, "Error: Issuer's name does not match the next certificate.\n");
+				exit(1);
+			}
+
+			fprintf( outfile, "\tVerification output: ");
+			print_verification_res( x509_cert_list[i-2], x509_cert_list[i-1], 
+				x509_crl_list, x509_ncrls);
+			fprintf( outfile, ".\n\n");
+
+		}
+
+
+		/* now we move ptr after the pem header 
+		 */
 		ptr = strstr(ptr, CERT_SEP);
 		if (ptr!=NULL)
 			ptr++;
@@ -1197,8 +1284,36 @@ int _verify_x509_mem( const char* cert, int cert_size)
 	/* The last certificate in the list will be used as
 	 * a CA (should be self signed).
 	 */
-	ret  = gnutls_x509_crt_list_verify( x509_cert_list, x509_ncerts,
-		&x509_cert_list[x509_ncerts-1], 1, x509_crl_list, x509_ncrls, 0, &output);
+	name_size = sizeof(name);
+	ret = gnutls_x509_crt_get_dn( x509_cert_list[x509_ncerts-1], name, &name_size);
+	if (ret < 0) {
+		fprintf(stderr, "Error in get_dn: %s\n", gnutls_strerror(ret));
+		exit(1);
+	}
+
+	fprintf( outfile, "Certificate[%d]: %s\n", x509_ncerts-1, name);
+
+	/* print issuer 
+	 */
+	issuer_name_size = sizeof(issuer_name);
+	ret = gnutls_x509_crt_get_issuer_dn( x509_cert_list[x509_ncerts-1], issuer_name, &issuer_name_size);
+	if (ret < 0) {
+		fprintf(stderr, "Error in get_dn: %s\n", gnutls_strerror(ret));
+		exit(1);
+	}
+
+	fprintf( outfile, "\tIssued by: %s\n", name);
+			
+	if (strcmp( issuer_name, name) != 0) {
+		fprintf(stderr, "Error: The last certificate is not self signed.\n");
+		exit(1);
+	}
+
+	fprintf( outfile, "\tVerification output: ");
+	print_verification_res( x509_cert_list[x509_ncerts-1], x509_cert_list[x509_ncerts-1], 
+				x509_crl_list, x509_ncrls);
+
+	fprintf( outfile, ".\n\n");
 
 	for (i=0;i<x509_ncerts;i++) {
 		gnutls_x509_crt_deinit( x509_cert_list[i]);
@@ -1219,36 +1334,72 @@ int _verify_x509_mem( const char* cert, int cert_size)
 	return output;
 }
 
-static void print_verification_res( unsigned int x) 
+static void print_verification_res( gnutls_x509_crt crt, gnutls_x509_crt issuer,
+	gnutls_x509_crl *crl_list, int crl_list_size)
 {
-	fprintf(outfile,  "Verification output:\n");
-	if (x&GNUTLS_CERT_INVALID)
-		fprintf(outfile, "\tcertificate chain is invalid.\n");
-	else
-	 	fprintf(outfile, "\tcertificate chain is valid.\n");
+unsigned int output;
+int comma=0;
+int ret;
+time_t now = time(0);
 
-	if (x&GNUTLS_CERT_NOT_TRUSTED)
-		fprintf(outfile, "\tThe certificate chain was NOT verified.\n");
-	else
-		fprintf(outfile, "\tThe certificate chain was verified.\n");
+	ret = gnutls_x509_crt_verify( crt, &issuer, 1, 0, &output);
+	if (ret < 0) {
+		fprintf(stderr, "Error in verification: %s\n", gnutls_strerror(ret));
+		exit(1);
+	}
 
-	if (x&GNUTLS_CERT_CORRUPTED)
-		fprintf(outfile, "\tA certificate is corrupt.\n");
+	if (output&GNUTLS_CERT_NOT_TRUSTED) {
+		fprintf(outfile, "Not verified");
+		comma = 1;
+	} else {
+		fprintf(outfile, "Verified");
+		comma = 1;
+	}
+
+	if (output&GNUTLS_CERT_SIGNER_NOT_CA) {
+		if (comma) fprintf(outfile, ", ");
+		fprintf(outfile, "Issuer is not a CA");
+		comma = 1;
+	}
+
+	/* Check expiration dates.
+	 */
+	
+	if (gnutls_x509_crt_get_activation_time(crt) > now) {
+		if (comma) fprintf(outfile, ", ");
+		comma = 1;
+		fprintf(outfile, "Not activated");
+	}
+	
+	if (gnutls_x509_crt_get_expiration_time(crt) < now) {
+		if (comma) fprintf(outfile, ", ");
+		comma = 1;
+		fprintf(outfile, "Expired");
+	}
+	
+	ret = gnutls_x509_crt_check_revocation( crt, crl_list, crl_list_size);
+	if (ret < 0) {
+		fprintf(stderr, "Error in verification: %s\n", gnutls_strerror(ret));
+		exit(1);
+	}
+	
+	if (ret == 1) { /* revoked */
+		if (comma) fprintf(outfile, ", ");
+		comma = 1;
+		fprintf(outfile, "Revoked");
+	}
+	
 
-	if (x&GNUTLS_CERT_REVOKED)
-		fprintf(outfile, "\tA certificate has been revoked.\n");
 }
 
 void verify_chain( void)
 {
-unsigned int output;
 size_t size;
 
 	size = fread( buffer, 1, sizeof(buffer)-1, infile);
 
-	output = _verify_x509_mem( buffer, size);
+	_verify_x509_mem( buffer, size);
 	
-	print_verification_res( output);
 }
 
 #include <gnutls/pkcs12.h>
@@ -1265,10 +1416,11 @@ void generate_pkcs12( void)
 	gnutls_datum data;
 	char* password;
 	const char* name;
+	unsigned int flags;
 	gnutls_datum key_id;
 	unsigned char _key_id[20];
 	int index;
-	
+
 	fprintf(stderr, "Generating a PKCS #12 structure...\n");
 	
 	key = load_private_key();
@@ -1278,8 +1430,9 @@ void generate_pkcs12( void)
 		name = read_str("Enter a name for the key: ");
 	} while( name == NULL);
 
-	password = getpass( "Enter password: ");
-	
+	if (info.pass) password = info.pass;
+	else password = getpass( "Enter password: ");
+
 	result = gnutls_pkcs12_bag_init( &bag);
 	if (result < 0) {
 		fprintf(stderr, "bag_init: %s\n", gnutls_strerror(result));
@@ -1296,21 +1449,12 @@ void generate_pkcs12( void)
 	key_id.data = _key_id;
 	key_id.size = size;
 		
-	size = sizeof(buffer);
-	result = gnutls_x509_crt_export( crt, GNUTLS_X509_FMT_DER, buffer, &size);
+	result = gnutls_pkcs12_bag_set_crt( bag, crt);
 	if (result < 0) {
-		fprintf(stderr, "crt_export: %s\n", gnutls_strerror(result));
+		fprintf(stderr, "set_crt: %s\n", gnutls_strerror(result));
 		exit(1);
 	}
 
-	data.data = buffer;
-	data.size = size;
-	result = gnutls_pkcs12_bag_set_data( bag, GNUTLS_BAG_CERTIFICATE, &data);
-	if (result < 0) {
-		fprintf(stderr, "bag_set_data: %s\n", gnutls_strerror(result));
-		exit(1);
-	}
-	
 	index = result;
 
 	result = gnutls_pkcs12_bag_set_friendly_name( bag, index, name);
@@ -1326,7 +1470,10 @@ void generate_pkcs12( void)
 		exit(1);
 	}
 
-	result = gnutls_pkcs12_bag_encrypt( bag, password, 0);
+	if (info.export) flags = GNUTLS_PKCS_USE_PKCS12_RC2_40;
+	else flags = GNUTLS_PKCS8_USE_PKCS12_3DES;
+
+	result = gnutls_pkcs12_bag_encrypt( bag, password, flags);
 	if (result < 0) {
 		fprintf(stderr, "bag_encrypt: %s\n", gnutls_strerror(result));
 		exit(1);
@@ -1340,9 +1487,12 @@ void generate_pkcs12( void)
 		exit(1);
 	}
 
+	if (info.export) flags = GNUTLS_PKCS_USE_PKCS12_RC2_40;
+	else flags = GNUTLS_PKCS_USE_PKCS12_3DES;
+
 	size = sizeof(buffer);
 	result = gnutls_x509_privkey_export_pkcs8( key, GNUTLS_X509_FMT_DER, password,
-		GNUTLS_PKCS8_USE_PKCS12_3DES, buffer, &size);
+		flags, buffer, &size);
 	if (result < 0) {
 		fprintf(stderr, "key_export: %s\n", gnutls_strerror(result));
 		exit(1);
@@ -1431,9 +1581,9 @@ void print_bag_data(gnutls_pkcs12_bag bag)
 {
 int result;
 int count, i, type;
-gnutls_datum data;
+gnutls_const_datum cdata;
 const char* str;
-gnutls_datum out;
+gnutls_datum out, data;
 
 	count = gnutls_pkcs12_bag_get_count( bag);
 	if (count < 0) {
@@ -1452,7 +1602,7 @@ gnutls_datum out;
 
 		fprintf( outfile, "\tType: %s\n", BAGTYPE( type));
 		
-		result = gnutls_pkcs12_bag_get_data( bag, i, &data);
+		result = gnutls_pkcs12_bag_get_data( bag, i, &cdata);
 		if (result < 0) {
 			fprintf(stderr, "get_data: %s\n", gnutls_strerror(result));
 			exit(1);
@@ -1477,6 +1627,12 @@ gnutls_datum out;
 			str = NULL;
 		}		
 	
+		/* we have to cast gnutls_const_datum to a
+		 * plain datum.
+		 */
+		data.data = (unsigned char*)cdata.data;
+		data.size = cdata.size;
+
 		if (str != NULL) {
 			gnutls_pem_base64_encode_alloc( str, &data, &out);
 			fprintf( outfile, "%s\n", out.data);
@@ -1503,7 +1659,8 @@ void pkcs12_info( void)
 	data.data = buffer;
 	data.size = size;
 
-	password = getpass( "Enter password: ");
+	if (info.pass) password = info.pass;
+	else password = getpass( "Enter password: ");
 	
 	result = gnutls_pkcs12_init(&pkcs12);
 	if (result < 0) {
@@ -1583,3 +1740,38 @@ void pkcs12_info( void)
 	
 }
 
+#else /* ENABLE_PKI */
+
+#include <stdio.h>
+
+int main (int argc, char **argv)
+{
+    printf ("\nX.509 PKI not supported. This program is a dummy.\n\n");
+    return 1;
+};
+
+#endif
+
+void certtool_version(void)
+{
+	fprintf(stderr, "certtool, ");
+	fprintf(stderr, "version %s. Libgnutls %s.\n", LIBGNUTLS_VERSION,
+		gnutls_check_version(NULL));
+}
+
+void print_license(void)
+{
+	fprintf(stdout,
+		"\nCopyright (C) 2001-2003 Nikos Mavroyanopoulos\n"
+		"This program is free software; you can redistribute it and/or modify \n"
+		"it under the terms of the GNU General Public License as published by \n"
+		"the Free Software Foundation; either version 2 of the License, or \n"
+		"(at your option) any later version. \n" "\n"
+		"This program is distributed in the hope that it will be useful, \n"
+		"but WITHOUT ANY WARRANTY; without even the implied warranty of \n"
+		"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \n"
+		"GNU General Public License for more details. \n" "\n"
+		"You should have received a copy of the GNU General Public License \n"
+		"along with this program; if not, write to the Free Software \n"
+		"Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n");
+}
