@@ -670,16 +670,17 @@ static void init_global_tls_stuff(void)
 {
 	int ret;
 
-	if (gnutls_global_init() < 0) {
-		fprintf(stderr, "global state initialization error\n");
+	if ((ret=gnutls_global_init()) < 0) {
+		fprintf(stderr, "global_init: %s\n", gnutls_strerror(ret));
 		exit(1);
 	}
+
 	gnutls_global_set_log_function(tls_log_func);
 	gnutls_global_set_log_level(debug);
 
-	if (gnutls_global_init_extra() < 0) {
+	if ((ret=gnutls_global_init_extra()) < 0) {
 		fprintf(stderr,
-			"global state (extra) initialization error\n");
+			"global_init_extra: %s\n", gnutls_strerror(ret));
 		exit(1);
 	}
 
