@@ -1,3 +1,4 @@
+#include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -338,22 +339,26 @@ void print_cert_vrfy(gnutls_session session)
 		return;
 	}
 	if (status < 0) {
-		printf("- Could not verify certificate (err %d)\n",
-		       status);
+		printf("- Could not verify certificate (err: %s)\n",
+		       gnutls_strerror(status));
 		return;
 	}
 
-	if (status & GNUTLS_CERT_INVALID)
-		printf("- Peer's certificate chain is broken\n");
-	if (status & GNUTLS_CERT_NOT_TRUSTED)
-		printf("- Peer's certificate is NOT trusted\n");
-	else
-		printf("- Peer's certificate is trusted\n");
-	if (status & GNUTLS_CERT_CORRUPTED)
-		printf("- Peer's certificate is corrupted\n");
-		
-	
-
+	if (gnutls_certificate_type_get(session)==GNUTLS_CRT_X509) {
+		if (status & GNUTLS_CERT_SIGNER_NOT_FOUND)
+			printf("- Peer's certificate issuer is unknown\n");
+		if (status & GNUTLS_CERT_INVALID)
+			printf("- Peer's certificate is NOT trusted\n");
+		else
+			printf("- Peer's certificate is trusted\n");
+	} else {
+		if (status & GNUTLS_CERT_INVALID)
+			printf("- Peer's key is invalid\n");
+		else
+			printf("- Peer's key is valid\n");
+		if (status & GNUTLS_CERT_SIGNER_NOT_FOUND)
+			printf("- Could not find a signer of the peer's key\n");
+	}
 }
 
 int print_info(gnutls_session session, const char* hostname)
@@ -405,7 +410,7 @@ int print_info(gnutls_session session, const char* hostname)
 		print_cert_info(session, hostname);
 
 		print_cert_vrfy(session);
-
+ 
 		/* Check if we have been using ephemeral Diffie Hellman.
 		 */
 		if (kx == GNUTLS_KX_DHE_RSA || kx == GNUTLS_KX_DHE_DSS) {
@@ -501,7 +506,7 @@ void print_list(void)
 void print_license(void)
 {
 	fprintf(stdout,
-		"\nCopyright (C) 2001-2002 Nikos Mavroyanopoulos\n"
+		"\nCopyright (C) 2001-2003 Nikos Mavroyanopoulos\n"
 		"This program is free software; you can redistribute it and/or modify \n"
 		"it under the terms of the GNU General Public License as published by \n"
 		"the Free Software Foundation; either version 2 of the License, or \n"
