@@ -1,5 +1,5 @@
 /*
- *      Copyright (C) 2001 Nikos Mavroyanopoulos
+ * Copyright (C) 2001,2003 Nikos Mavroyanopoulos
  *
  * This file is part of GNUTLS.
  *
@@ -428,11 +428,19 @@ int main(int argc, char **argv)
 {
 	gaainfo info;
 	char *passwd;
-	int salt;
+	int salt, ret;
 	struct passwd *pwd;
 
-	gnutls_global_init();
-	gnutls_global_init_extra();
+	if ((ret=gnutls_global_init()) < 0) {
+		fprintf(stderr, "global_init: %s\n", gnutls_strerror(ret));
+		exit(1);
+	}
+
+	if ((ret=gnutls_global_init_extra()) < 0) {
+		fprintf(stderr,
+			"global_init_extra: %s\n", gnutls_strerror(ret));
+		exit(1);
+	}
 	
 	if (gaa(argc, argv, &info) != -1) {
 		fprintf(stderr, "Error in the arguments.\n");
