@@ -18,6 +18,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#include <config.h>
 #include <gnutls/gnutls.h>
 #include <gnutls/extra.h>
 #include <gnutls/x509.h>
@@ -288,7 +289,7 @@ int old;
 
 int test_aes( gnutls_session session) {
 int ret;
-	ADD_CIPHER(session, GNUTLS_CIPHER_RIJNDAEL_128_CBC);
+	ADD_CIPHER(session, GNUTLS_CIPHER_AES_128_CBC);
 	ADD_ALL_COMP(session);
 	ADD_ALL_CERTTYPES(session);
 	ADD_ALL_PROTOCOLS(session);
@@ -321,7 +322,7 @@ int ret;
 
 int test_unknown_ciphersuites( gnutls_session session) {
 int ret;
-	ADD_CIPHER3(session, GNUTLS_CIPHER_RIJNDAEL_128_CBC,
+	ADD_CIPHER3(session, GNUTLS_CIPHER_AES_128_CBC,
 		GNUTLS_CIPHER_3DES_CBC, GNUTLS_CIPHER_ARCFOUR_128);
 	ADD_ALL_COMP(session);
 	ADD_ALL_CERTTYPES(session);
@@ -589,7 +590,8 @@ int ret;
 }
 #endif
 
-int test_session_resume2( gnutls_session session) {
+int test_session_resume2( gnutls_session session) 
+{
 int ret;
 char tmp_session_id[32];
 int tmp_session_id_size;
@@ -602,6 +604,8 @@ int tmp_session_id_size;
 	ADD_ALL_PROTOCOLS(session);
 	ADD_ALL_MACS(session);
 	ADD_ALL_KX(session);
+
+	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, xcred);
 	gnutls_credentials_set(session, GNUTLS_CRD_ANON, anon_cred);
 
 	gnutls_session_set_data(session, session_data, session_data_size);
@@ -611,12 +615,14 @@ int tmp_session_id_size;
 
 	ret = do_handshake( session);
 	if (ret == FAILED) return ret;
-	
+
 	/* check if we actually resumed the previous session */
 
 	session_id_size = sizeof(session_id);
 	gnutls_session_get_id(session, session_id, &session_id_size);
 
+	if (gnutls_session_is_resumed( session)) return SUCCEED;
+
 	if (memcmp(tmp_session_id, session_id, tmp_session_id_size) == 0)
 		return SUCCEED;
 	else
@@ -624,6 +630,8 @@ int tmp_session_id_size;
 
 }
 
+extern char* hostname;
+
 int test_certificate( gnutls_session session) {
 int ret;
 
@@ -640,7 +648,7 @@ int ret;
 	if (ret == FAILED) return ret;
 
 	printf("\n");
-	print_cert_info( session);
+	print_cert_info( session, hostname);
 
 	return SUCCEED;
 }
